@@ -27,6 +27,7 @@ $string['configtitle'] = 'Block title';
 $string['configtitleblankhides'] = 'Block title (no title if blank)';
 $string['leaveblanktohide'] = 'leave blank to hide the title';
 $string['mentees:addinstance'] = 'Add a new mentees block';
-$string['mentees:myaddinstance'] = 'Add a new mentees block to the My Moodle page';
+$string['mentees:myaddinstance'] = 'Add a new mentees block to Dashboard';
 $string['newmenteesblock'] = '(new Mentees block)';
 $string['pluginname'] = 'Mentees';
+$string['privacy:metadata'] = 'The Mentees block only shows data stored in other locations.';
