@@ -23,8 +23,8 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
-        'core/custom_interaction_events', 'core/modal_backdrop', 'core/modal_events'],
-     function($, Templates, Notification, KeyCodes, CustomEvents, ModalBackdrop, ModalEvents) {
+        'core/custom_interaction_events', 'core/modal_backdrop', 'core/event', 'core/modal_events'],
+     function($, Templates, Notification, KeyCodes, CustomEvents, ModalBackdrop, Event, ModalEvents) {
 
     var SELECTORS = {
         CONTAINER: '[data-region="modal-container"]',
@@ -50,6 +50,12 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      */
     var backdropPromise;
 
+    /**
+     * A counter that gets incremented for each modal created. This can be
+     * used to generate unique values for the modals.
+     */
+    var modalCounter = 0;
+
     /**
      * Constructor for the Modal.
      *
@@ -66,6 +72,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
         this.isAttached = false;
         this.bodyJS = null;
         this.footerJS = null;
+        this.modalCount = modalCounter++;
 
         if (!this.root.is(SELECTORS.CONTAINER)) {
             Notification.exception({message: 'Element is not a modal container'});
@@ -212,6 +219,16 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
         return this.footer;
     };
 
+    /**
+     * Get the unique modal count.
+     *
+     * @method getModalCount
+     * @return {int}
+     */
+    Modal.prototype.getModalCount = function() {
+        return this.modalCount;
+    };
+
     /**
      * Set the modal title element.
      *
@@ -224,13 +241,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
     Modal.prototype.setTitle = function(value) {
         var title = this.getTitle();
 
-        if (typeof value === 'string') {
-            title.html(value);
-        } else {
-            value.then(function(content) {
-                return title.html(content);
-            });
-        }
+        this.asyncSet(value, title.html.bind(title));
     };
 
     /**
@@ -248,31 +259,107 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
         if (typeof value === 'string') {
             // Just set the value if it's a string.
             body.html(value);
+            Event.notifyFilterContentUpdated(body);
+            this.getRoot().trigger(ModalEvents.bodyRendered, this);
         } else {
+            var jsPendingId = 'amd-modal-js-pending-id-' + this.getModalCount();
+            M.util.js_pending(jsPendingId);
             // Otherwise we assume it's a promise to be resolved with
             // html and javascript.
-            Templates.render(TEMPLATES.LOADING, {}).done(function(html) {
-                body.html(html);
+            var contentPromise = null;
+            body.css('overflow', 'hidden');
 
-                value.done(function(html, js) {
+            if (value.state() == 'pending') {
+                // We're still waiting for the body promise to resolve so
+                // let's show a loading icon.
+                body.animate({height: '100px'}, 150);
+
+                body.html('');
+                contentPromise = Templates.render(TEMPLATES.LOADING, {})
+                    .then(function(html) {
+                        var loadingIcon = $(html).hide();
+                        body.html(loadingIcon);
+                        loadingIcon.fadeIn(150);
+
+                        // We only want the loading icon to fade out
+                        // when the content for the body has finished
+                        // loading.
+                        return $.when(loadingIcon.promise(), value);
+                    })
+                    .then(function(loadingIcon) {
+                        // Once the content has finished loading and
+                        // the loading icon has been shown then we can
+                        // fade the icon away to reveal the content.
+                        return loadingIcon.fadeOut(100).promise();
+                    })
+                    .then(function() {
+                        return value;
+                    });
+            } else {
+                // The content is already loaded so let's just display
+                // it to the user. No need for a loading icon.
+                contentPromise = value;
+            }
+
+            // Now we can actually display the content.
+            contentPromise.then(function(html, js) {
+                var result = null;
+
+                if (this.isVisible()) {
+                    // If the modal is visible then we should display
+                    // the content gracefully for the user.
+                    body.css('opacity', 0);
+                    var currentHeight = body.innerHeight();
                     body.html(html);
+                    // We need to clear any height values we've set here
+                    // in order to measure the height of the content being
+                    // added. This then allows us to animate the height
+                    // transition.
+                    body.css('height', '');
+                    var newHeight = body.innerHeight();
+                    body.css('height', currentHeight + 'px');
+                    result = body.animate(
+                        {height: newHeight + 'px', opacity: 1},
+                        {duration: 150, queue: false}
+                    ).promise();
+                } else {
+                    // Since the modal isn't visible we can just immediately
+                    // set the content. No need to animate it.
+                    body.html(html);
+                }
 
-                    if (js) {
-                        if (this.isAttached) {
-                            // If we're in the DOM then run the JS immediately.
-                            Templates.runTemplateJS(js);
-                        } else {
-                            // Otherwise cache it to be run when we're attached.
-                            this.bodyJS = js;
-                        }
+                if (js) {
+                    if (this.isAttached) {
+                        // If we're in the DOM then run the JS immediately.
+                        Templates.runTemplateJS(js);
+                    } else {
+                        // Otherwise cache it to be run when we're attached.
+                        this.bodyJS = js;
                     }
-                }.bind(this));
-            }.bind(this));
+                }
+                Event.notifyFilterContentUpdated(body);
+                this.getRoot().trigger(ModalEvents.bodyRendered, this);
+
+                return result;
+            }.bind(this))
+            .fail(Notification.exception)
+            .always(function() {
+                // When we're done displaying all of the content we need
+                // to clear the custom values we've set here.
+                body.css('height', '');
+                body.css('overflow', '');
+                body.css('opacity', '');
+                M.util.js_complete(jsPendingId);
+
+                return;
+            })
+            .fail(Notification.exception);
         }
     };
 
     /**
-     * Set the modal footer element.
+     * Set the modal footer element. The footer element is made visible, if it
+     * isn't already.
      *
      * This method is overloaded to take either a string
      * value for the body or a jQuery promise that is resolved with HTML and Javascript
@@ -282,6 +369,9 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      * @param {(string|object)} value The footer string or jQuery promise
      */
     Modal.prototype.setFooter = function(value) {
+        // Make sure the footer is visible.
+        this.showFooter();
+
         var footer = this.getFooter();
 
         if (typeof value === 'string') {
@@ -310,6 +400,34 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
         }
     };
 
+    /**
+     * Check if the footer has any content in it.
+     *
+     * @method hasFooterContent
+     * @return {bool}
+     */
+    Modal.prototype.hasFooterContent = function() {
+        return this.getFooter().children().length ? true : false;
+    };
+
+    /**
+     * Hide the footer element.
+     *
+     * @method hideFooter
+     */
+    Modal.prototype.hideFooter = function() {
+        this.getFooter().addClass('hidden');
+    };
+
+    /**
+     * Show the footer element.
+     *
+     * @method showFooter
+     */
+    Modal.prototype.showFooter = function() {
+        this.getFooter().removeClass('hidden');
+    };
+
     /**
      * Mark the modal as a large modal.
      *
@@ -320,7 +438,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             return;
         }
 
-        this.getRoot().addClass('large');
+        this.getModal().addClass('modal-lg');
     };
 
     /**
@@ -330,7 +448,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      * @return {bool}
      */
     Modal.prototype.isLarge = function() {
-        return this.getRoot().hasClass('large');
+        return this.getModal().hasClass('modal-lg');
     };
 
     /**
@@ -343,7 +461,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             return;
         }
 
-        this.getRoot().removeClass('large');
+        this.getModal().removeClass('modal-lg');
     };
 
     /**
@@ -353,7 +471,7 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      * @return {bool}
      */
     Modal.prototype.isSmall = function() {
-        return !this.getRoot().hasClass('large');
+        return !this.getModal().hasClass('modal-lg');
     };
 
     /**
@@ -422,6 +540,12 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             return;
         }
 
+        if (this.hasFooterContent()) {
+            this.showFooter();
+        } else {
+            this.hideFooter();
+        }
+
         if (!this.isAttached) {
             this.attachToDOM();
         }
@@ -448,10 +572,6 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      * @method hide
      */
     Modal.prototype.hide = function() {
-        if (!this.isVisible()) {
-            return;
-        }
-
         this.getBackdrop().done(function(backdrop) {
             if (!this.countOtherVisibleModals()) {
                 // Hide the backdrop if we're the last open modal.
@@ -600,5 +720,30 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
         }.bind(this));
     };
 
+    /**
+     * Set or resolve and set the value using the function.
+     *
+     * @method asyncSet
+     * @param {(string|object)} value The string or jQuery promise.
+     * @param {function} setFunction The setter
+     * @return {Promise}
+     */
+    Modal.prototype.asyncSet = function(value, setFunction) {
+        var p = value;
+        if (typeof value !== 'object' || !value.hasOwnProperty('then')) {
+            p = $.Deferred();
+            p.resolve(value);
+        }
+
+        p.then(function(content) {
+            setFunction(content);
+
+            return;
+        })
+        .fail(Notification.exception);
+
+        return p;
+    };
+
     return Modal;
 });
