@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -18,15 +17,16 @@
 /**
  * Strings for component 'chat', language 'en', branch 'MOODLE_20_STABLE'
  *
- * @package   chat
+ * @package   mod_chat
  * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-$string['activityoverview'] = 'You have upcoming chat';
+$string['activityoverview'] = 'You have upcoming chat sessions';
 $string['ajax'] = 'Version using Ajax';
 $string['autoscroll'] = 'Auto scroll';
-$string['beep'] = 'beep';
+$string['beep'] = 'Beep';
+$string['bubble'] = 'Bubble';
 $string['cantlogin'] = 'Could not log in to chat room!!';
 $string['composemessage'] = 'Compose a message';
 $string['configmethod'] = 'The ajax chat method provide an ajax based chat interface, it contacts server regularly for update. The normal chat method involves the clients regularly contacting the server for updates. It requires no configuration and works everywhere, but it can create a large load on the server with many chatters.  Using a server daemon requires shell access to Unix, but it results in a fast scalable chat environment.';
@@ -38,6 +38,8 @@ $string['configserverhost'] = 'The hostname of the computer where the server dae
 $string['configserverip'] = 'The numerical IP address that matches the above hostname';
 $string['configservermax'] = 'Max number of clients allowed';
 $string['configserverport'] = 'Port to use on the server for the daemon';
+$string['compact'] = 'Compact';
+$string['coursetheme'] = 'Course theme';
 $string['currentchats'] = 'Active chat sessions';
 $string['currentusers'] = 'Current users';
 $string['deletesession'] = 'Delete this session';
@@ -54,14 +56,21 @@ $string['chat:deletelog'] = 'Delete chat logs';
 $string['chat:exportparticipatedsession'] = 'Export chat session which you took part in';
 $string['chat:exportsession'] = 'Export any chat session';
 $string['chat:chat'] = 'Access a chat room';
-$string['chatintro'] = 'Introduction text';
+$string['chatintro'] = 'Description';
 $string['chatname'] = 'Name of this chat room';
-$string['chat:readlog'] = 'Read chat logs';
+$string['chat:readlog'] = 'View chat logs';
 $string['chatreport'] = 'Chat sessions';
 $string['chat:talk'] = 'Talk in a chat';
 $string['chattime'] = 'Next chat time';
+$string['chat:view'] = 'View chat activity';
 $string['entermessage'] = "Enter your message";
+$string['eventmessagesent'] = 'Message sent';
+$string['eventsessionsviewed'] = 'Sessions viewed';
 $string['idle'] = 'Idle';
+$string['indicator:cognitivedepth'] = 'Chat cognitive';
+$string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in a Chat activity.';
+$string['indicator:socialbreadth'] = 'Chat social';
+$string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in a Chat activity.';
 $string['inputarea'] = 'Input area';
 $string['invalidid'] = 'Could not find that chat room!';
 $string['list_all_sessions'] = 'List all sessions.';
@@ -109,16 +118,31 @@ $string['page-mod-chat-x'] = 'Any chat module page';
 $string['pastchats'] = 'Past chat sessions';
 $string['pluginadministration'] = 'Chat administration';
 $string['pluginname'] = 'Chat';
+$string['privacy:metadata:chat_messages_current'] = 'Current chat session. This data is temporary and is deleted after the chat session is deleted';
+$string['privacy:metadata:chat_users'] = 'Keeps track of which users are in which chat rooms';
+$string['privacy:metadata:chat_users:firstping'] = 'Time of the first access to chat room';
+$string['privacy:metadata:chat_users:ip'] = 'User IP';
+$string['privacy:metadata:chat_users:lang'] = 'User language';
+$string['privacy:metadata:chat_users:lastmessageping'] = 'Time of the last message in this chat room';
+$string['privacy:metadata:chat_users:lastping'] = 'Time of the last access to chat room';
+$string['privacy:metadata:chat_users:userid'] = 'The user ID';
+$string['privacy:metadata:chat_users:version'] = 'How user accessed the chat (sockets/basic/ajax/header_js)';
+$string['privacy:metadata:messages'] = 'A record of the messages sent during a chat session';
+$string['privacy:metadata:messages:issystem'] = 'Whether the message is a system-generated message';
+$string['privacy:metadata:messages:message'] = 'The message';
+$string['privacy:metadata:messages:timestamp'] = 'The time when the message was sent.';
+$string['privacy:metadata:messages:userid'] = 'The user ID of the author of the message';
 $string['refreshroom'] = 'Refresh room';
 $string['refreshuserlist'] = 'Refresh user list';
 $string['removemessages'] = 'Remove all messages';
 $string['repeatdaily'] = 'At the same time every day';
 $string['repeatnone'] = 'No repeats - publish the specified time only';
-$string['repeattimes'] = 'Repeat sessions';
+$string['repeattimes'] = 'Repeat/publish session times';
 $string['repeatweekly'] = 'At the same time every week';
 $string['saidto'] = 'said to';
 $string['savemessages'] = 'Save past sessions';
 $string['seesession'] = 'See this session';
+$string['search:activity'] = 'Chat - activity information';
 $string['send'] = 'Send';
 $string['sending'] = 'Sending';
 $string['serverhost'] = 'Server name';
@@ -126,7 +150,7 @@ $string['serverip'] = 'Server ip';
 $string['servermax'] = 'Max users';
 $string['serverport'] = 'Server port';
 $string['sessions'] = 'Chat sessions';
-$string['sessionstart'] = 'Chat session will be start in: {$a}';
+$string['sessionstart'] = 'The next chat session will start on {$a->date}, ({$a->fromnow} from now)';
 $string['strftimemessage'] = '%H:%M';
 $string['studentseereports'] = 'Everyone can view past sessions';
 $string['studentseereports_help'] = 'If set to No, only users have mod/chat:readlog capability are able to see the chat logs';
