@@ -302,7 +302,7 @@ $string['presetinfo'] = 'Saving as a preset will publish this template. Other us
 $string['presets'] = 'Presets';
 $string['privacy:metadata:commentpurpose'] = 'Comments on database records';
 $string['privacy:metadata:data_content'] = 'Represents one answer to one field in database activity module';
-$string['privacy:metadata:data_content:fieldid'] = 'Field definition id';
+$string['privacy:metadata:data_content:fieldid'] = 'Field definition ID';
 $string['privacy:metadata:data_content:content'] = 'Content';
 $string['privacy:metadata:data_content:content1'] = 'Additional content 1';
 $string['privacy:metadata:data_content:content2'] = 'Additional content 2';
